
#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void ATR (const MSXDataRec * MasterData, const MSXDataInfoRec * Periods, 
				MSXDataInfoRec * Result)

{	
	MSXDataInfoRec Temp;

	// Return an invalid array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &Temp) )
	{
		Result->iFirstValid = 0; 
		Result->iLastValid = 0;
		return;
	}

	TrueRange(MasterData, &Temp);
	
	SMA(&Temp, Periods, Result);

	int startIdx = Result->iFirstValid;
	int endIdx = Result->iLastValid;

	for (int i = startIdx+1; i <= endIdx; i++)
	{
		// we are assuming good period behavior in the beginning...
		int period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;

		Result->pfValue[i] = (float)ForceFloatRange( 
							( Result->pfValue[i-1] * (period-1) + Temp.pfValue[i] ) / period
							); 
	}

	// Don't Forget to DeAllocate Array
	MSX_ARRAY_DEALLOC(&Temp);
}
