/* 

/// Brad and Mike Adaptive Function:  AroonDown 

	// This function uses a faster algorithm like the one in SMA_Fast

	// NOTE:  I believe that in the case of a tie
*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void AroonDown (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array
	int j; // the looping index for the periods array.

	int curStartPt; // the starting point calculated from the current main index and the current periods
	int prevStartPt; // the starting point calculated from the prev main index and the prev periods

	int period;	// Set to be the requested number of lookback indicies each time.
	int changePeriods; // The change (+ or -) of the sliding back edge index of 
						// the window each time through the main loop.
	
	Result->iFirstValid = startIdx; 
	Result->iLastValid = endIdx;

	int LowestIdx   =  -1; 
	float lowest = FLT_MAX; // make it the largest float so any other number is less than it
	float tmp;

	curStartPt = startIdx;
   
	for (i = startIdx; i <= endIdx; i++ )
	{
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		prevStartPt = curStartPt;
		curStartPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		// see if we are adding or subtracting elements this time.
		changePeriods = curStartPt - prevStartPt;

		// Get the newest element at the front of the "window"
		tmp = Input->pfValue[i];


		// initialize to the previous high. there are a few? cases where none of the Result
		Result->pfValue[i] = (float)(i - LowestIdx); 

		// *** Improvement over TA_LIB method.
		// I check for the new element being the highest before anything else,
		// because if it is a new max, then we don't need to look back for a new one
		if( tmp <= lowest )
		{
			LowestIdx = i;
			lowest = tmp;

			Result->pfValue[i] = (float)(i - LowestIdx); 
		}
		else
		{
			// We might have to look for a new minimum if we are throwing away elements.
			if (changePeriods > 0)
			{
				// But Only if the Highest Element was in the part that we threw away. (and was not the newest element)
				if( LowestIdx < curStartPt)
				{
					LowestIdx = curStartPt;
					lowest = Input->pfValue[LowestIdx];
        
					// loop through the entire current lookback and find a new max, worst case.
					for(j = curStartPt + 1; j <= i; j++)
					{
					   tmp = Input->pfValue[j];
					   if( tmp <= lowest )
					   {
						  LowestIdx = j;
						  lowest = tmp;
					   }
					}
					Result->pfValue[i] = (float)(i - LowestIdx); 
				}
			}
		}

		// If we are adding elements, we just loop through them all and see if each is a new minimum
		// (no matter if we found a new min on the front end or not)
		if (changePeriods < 0)
		{
			for (int j = 0; j > changePeriods; j--)
			{
				tmp = Input->pfValue[prevStartPt + j - 1];
				if (tmp < lowest)
				{
					LowestIdx = prevStartPt + j - 1;
					lowest = tmp;
					Result->pfValue[i] = (float)(i - LowestIdx); 
				}
			}
		}  

		// result at this point is HHVBarsAgo.

		Result->pfValue[i] = (float)ForceFloatRange(
				
			(period - Result->pfValue[i]) / period * 100

		);
		
	}
}
   
   
