/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
/* 

/// Brad and Mike Adaptive Function:  BBandBot (Bollinger Band Bottom) )

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void BBandBot (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
		   const MSXDataInfoRec * Periods, const float deviations, MSXDataInfoRec * Result)
{
	// Stores a temporary Result from a Call to the EMA function,
	MSXDataInfoRec Temp;

	// Return an empty array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &Temp) )
	{
		Result->iFirstValid = -1; 
		Result->iLastValid = 0;
		return;
	}

	// Call the EMA function and then pass the output of first as input to second.
	// remember that the valid result value range is determined by the last function call, StdDev
	// (it is NOT the sum of the valid indicies of the two indicators)
	EMA(Input, Periods, &Temp);
	Stdev(MasterData, Input, Periods, deviations, Result);

	for(int j = Result->iFirstValid; j <= Result->iLastValid ; j++ )
		Result->pfValue[j] = (float)ForceFloatRange(Temp.pfValue[j] - Result->pfValue[j]) ;

	// Don't Forget to DeAllocate Array
	MSX_ARRAY_DEALLOC(&Temp);
}
