/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
/* 

/// Brad and Mike Adaptive Function:  CCI (Commodity Channel Index)

  	//  12-20-04 - Non Optomized, mainly just Umit and Steve's Code

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "Adaptive.h"
#include "MSXSafetyChecks.h"
#include "MSXStruc.h"  // required definitions


void CCI(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods, 
			 MSXDataInfoRec * Result)
{
	int l_iIndex = MasterData->sClose.iFirstValid;
	int l_iMaxIndex = MasterData->sClose.iLastValid;
	MSXDataInfoRec l_psTP;
	l_psTP.iFirstValid = l_iIndex;
	l_psTP.iLastValid = l_iMaxIndex;
	l_psTP.pfValue = new float[l_iMaxIndex+1];
	MSXDataInfoRec l_psMD;
	l_psMD.iFirstValid = l_iIndex;
	l_psMD.iLastValid = l_iMaxIndex;
	l_psMD.pfValue = new float[l_iMaxIndex+1];
	int l_iPeriod = 0;
	double l_lfDbl = 0.0;

	
	Result->iFirstValid = l_iIndex;
	Result->iLastValid = l_iMaxIndex;


	for(int i=l_iIndex; i <= l_iMaxIndex; i++)
	{
		// Calculate the Typical price (H + L + C)/3
		l_psTP.pfValue[i] = (float)ForceFloatRange((MasterData->sHigh.pfValue[i] +
		MasterData->sLow.pfValue[i] + MasterData->sClose.pfValue[i]) / 3.0);
	}

	// Calculate the SMA
	SMA(&l_psTP, Periods, Result);

	// Calculate the Mean deviation
	for(i=l_iIndex; i <= l_iMaxIndex; i++)
	{
		l_iPeriod = (int)ForceIntRange(Periods->pfValue[i]);
		
		if(l_iPeriod < 1) 
			l_iPeriod = 1;

		if(l_iPeriod > i) 
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			l_iPeriod = i;
		}

		l_lfDbl = 0.0;
		for(int x=i; x > (i - l_iPeriod); x--)
		{
			l_lfDbl += fabs(Result->pfValue[i] - l_psTP.pfValue[x]);
		}

		l_psMD.pfValue[i] = float(ForceFloatRange(l_lfDbl/l_iPeriod));

		// Calculate the CCI
		l_lfDbl = (0.015 * l_psMD.pfValue[i]);
		
		if(l_lfDbl == 0) 
			l_lfDbl = FLT_MIN;
		
		Result->pfValue[i] = float(ForceFloatRange((l_psTP.pfValue[i] - Result->pfValue[i]) / l_lfDbl));
	
	}

	delete[] l_psTP.pfValue;
	delete[] l_psMD.pfValue;
}
