/* 

/// Brad and Mike Adaptive Function:  DEMA(Double Exponential Moving Average) )

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void DEMA (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
		   const MSXDataInfoRec * Periods, MSXDataInfoRec * Result)
{
	// Stores a temporary Result from a Call to the EMA function,
	MSXDataInfoRec Temp;

	// Return an empty array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &Temp) )
	{
		Result->iFirstValid = -1; 
		Result->iLastValid = 0;
		return;
	}

	// Call the EMA function and then pass the output of first as input to second.
	// We can use the results array as a temp array.
	EMA(Input, Periods, &Temp);
	EMA(&Temp, Periods, Result);

	// Use the DEMA formula, where Temp = EMA(t,periods), and Result(as input) = EMA(EMA(t,periods),periods).
	for(int j = Result->iFirstValid; j <= Result->iLastValid ; j++ )
		Result->pfValue[j] = (float)ForceFloatRange((2.0 * Temp.pfValue[j]) - Result->pfValue[j]);
	
	// Since the Result Array was the last one returned from another function,
	// It contains the correct FirstValid and LastValid values.

	// Don't Forget to DeAllocate Array
	MSX_ARRAY_DEALLOC(&Temp);
}
