#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void ForecastOsc (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
			MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array
	int j; // the looping index for the periods array.

	int curStartPt; // Set to be the starting point (index) in the array each time.
	int period;  // Set to be the IDEAL number of lookback indicies each time
	int n;  // Set to be the ACTUAL number of lookback indicies each time.
	
	double SumX;
	double SumXSqr;
	double Divisor;
	double SumY;
	double SumXY;
	double m;
	double b;
	double tempValue1;

	Result->iFirstValid = startIdx;
	Result->iLastValid = endIdx;

	// Main Loop. 
	for (i = startIdx; i <= endIdx; i++)
	{	
		period = int(ForceIntRange(Periods->pfValue[i]));

		// always use this convention, no such thing as negative or zero period
		if(period < 1) 
			period = 1;
		
		curStartPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		SumXY = 0;
		SumY = 0;
		n = i - curStartPt + 1; // ACTUAL number of periods calculated, not REQUESTED number

		for( j = n; j--; )
		{
			SumY += tempValue1 = Input->pfValue[i - j];
			SumXY += (double)j * tempValue1;
		}

		SumX = n * ( n - 1 ) * 0.5;
		SumXSqr = n * ( n - 1 ) * ( 2 * n - 1 ) / 6;
		Divisor = SumX * SumX - n * SumXSqr;

		if (!IsDoubleZero(Divisor))
			m = ( n * SumXY - SumX * SumY) / Divisor;
		else
			m = 0.0;

		b = ( SumY - m * SumX ) / n;
	
		if (IsDoubleZero(Input->pfValue[i]))
			Result->pfValue[i] = 0.0;
		else
			Result->pfValue[i] = (float)ForceFloatRange(
			
						 (Input->pfValue[i] - (m*n + b)) / Input->pfValue[i]  * 100
					// using this method instead of one above gives slightly different but
					// still wrong results
						//(Input->pfValue[i] - (m*n + b)) / (m*n + b)  * 100
						); 
	
	}
}
