/* 

/// Brad and Mike Adaptive Function:  MACD (Moving Average Convergence/Divergence) )

	// This function could be spead up by calculating
*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void MACD (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * slowPeriods, 
			const MSXDataInfoRec * fastPeriods, MSXDataInfoRec * Result)
{
	// Stores a temporary Result from a Call to the EMA function,
	MSXDataInfoRec Temp;

	// Return an empty array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &Temp) )
	{
		Result->iFirstValid = -1; 
		Result->iLastValid = 0;
		return;
	}

	// Call the slow and fast EMA, using Temp and the result array as temporary arrays.
	EMA(Input, slowPeriods, Result);
	EMA(Input, fastPeriods, &Temp);

	// Subtract the slower moving average from the faster one.
	for(int j = Result->iFirstValid; j <= Result->iLastValid ; j++ )
		Result->pfValue[j] = (float)ForceFloatRange(Temp.pfValue[j] - Result->pfValue[j]);

	// Since the Result Array was the one used in the slowest EMA, it contains the correct
	// first and last valid values.

	// Don't Forget to DeAllocate Array
	MSX_ARRAY_DEALLOC(&Temp);

}