/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void Mid (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
		  const MSXDataInfoRec * Periods, MSXDataInfoRec * Result)
{
	// create temp variable on the stack, then we allocate its array
	// dynamically, and destroy it when we return.
	MSXDataInfoRec TempHHV;
	
	// Return an invalid array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &TempHHV) )
	{
		Result->iFirstValid = 0; 
		Result->iLastValid = 0;
		return;
	}
	

	// NOTE : we are using Result as temp array here.
	HHV(Input, Periods, &TempHHV);
	LLV(Input, Periods, Result);
	
	int startIdx = Result->iFirstValid;
	int endIdx = Result->iLastValid;

	for (int i = startIdx; i <= endIdx; i++)
	{

		Result->pfValue[i] = (float)ForceFloatRange(
			
			Result->pfValue[i] + ((TempHHV.pfValue[i] - Result->pfValue[i]) / 2.0 )
			
			); 
	}

	MSX_ARRAY_DEALLOC(&TempHHV);
}
