#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void PriceChannelHigh (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid+1;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array
	int j; // the looping index for the periods array.

	int curStartPt; // the starting point calculated from the current main index and the current periods
	int prevStartPt; // the starting point calculated from the prev main index and the prev periods

	int period;	// Set to be the requested number of lookback indicies each time.
	int changePeriods; // The change (+ or -) of the sliding back edge index of 
						// the window each time through the main loop.
	
	Result->iFirstValid = startIdx; 
	Result->iLastValid = endIdx;

	int HighestIdx   =  -1; 
	float highest = -FLT_MAX; // make it the largest float so any other number is less than it
	float tmp;

	curStartPt = startIdx;
   
	// 3/11/05 - changed indicies to reflect using i+1
	for (i = startIdx - 1; i < endIdx; i++ )
	{
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		prevStartPt = curStartPt;
		curStartPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		// see if we are adding or subtracting elements this time.
		changePeriods = curStartPt - prevStartPt;

		// Get the newest element at the front of the "window"
		tmp = Input->pfValue[i];


		// initialize to the previous high. there are a few? cases where none of the Result
		Result->pfValue[i+1] = highest;

		// *** Improvement over TA_LIB method.
		// I check for the new element being the highest before anything else,
		// because if it is a new max, then we don't need to look back for a new one
		if( tmp >= highest )
		{
			HighestIdx = i;
			highest = tmp;

			Result->pfValue[i+1] = highest; 
		}
		else
		{
			// We might have to look for a new minimum if we are throwing away elements.
			if (changePeriods > 0)
			{
				// But Only if the Highest Element was in the part that we threw away. (and was not the newest element)
				if( HighestIdx < curStartPt)
				{
					HighestIdx = curStartPt;
					highest = Input->pfValue[HighestIdx];
        
					// loop through the entire current lookback and find a new max, worst case.
					for(j = curStartPt + 1; j <= i; j++)
					{
					   tmp = Input->pfValue[j];
					   if( tmp >= highest )
					   {
						  HighestIdx = j;
						  highest = tmp;
					   }
					}
					Result->pfValue[i+1] = highest;
				}
			}
		}

		// If we are adding elements, we just loop through them all and see if each is a new minimum
		// (no matter if we found a new min on the front end or not)
		if (changePeriods < 0)
		{
			for (int j = 0; j > changePeriods; j--)
			{
				tmp = Input->pfValue[prevStartPt + j - 1];
				if (tmp > highest)
				{
					HighestIdx = prevStartPt + j - 1;
					highest = tmp;
					Result->pfValue[i+1] = highest;
				}
			}
		}  
	}
}