#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void RMI (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				 const MSXDataInfoRec * MoPeriods, MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array
	int j; // the looping index for the periods array.

	int startPt; // Set to be the starting point (index) in the array each time.
	int period;  // Set to be the IDEAL number of lookback indicies each time
	int n;  // Set to be the ACTUAL number of lookback indicies each time.
	
	int moPeriod;

	double CU[2] = {0,0};
	double CD[2] = {0,0};

	double today;
	double yesterday;

	Result->iFirstValid = startIdx;
	Result->iLastValid = endIdx;

	// Main Loop. 
	for (i = startIdx; i <= endIdx; i++)
	{	
		period = int(ForceIntRange(Periods->pfValue[i]));

		moPeriod = int(ForceIntRange(MoPeriods->pfValue[i]));

		// always use this convention, no such thing as negative or zero period
		if(period < 1) 
			period = 1;

		if(moPeriod < 1) 
			moPeriod = 1;
		
		startPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (startPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i+1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			startPt = startIdx;
		}

		n = i - startPt + 1;

	
		// so we don't have to do extra range checking, we just set the moPeriod
		// equal to the regular period here, if it is greater than it.
		if (moPeriod > period)
			moPeriod = period;


		//if (n == 1)
		//	Result->pfValue[i] = 0.0;

		CU[0] = 0.0;
		CD[0] = 0.0;

		
		//first time through, n = 1, so we don't go through LOOP
		if (n == 1)
		{
			CU[0] = 0.0;
			CD[0] = 0.0;
		}
		
		// We only do an average on the first time! (we really don't even have to average, since
		// the calculations are relative, so we avoid the uneccessary division.
		else if (n == 2)
		{
			for (j = n; j >= 2; j--)
			{
				today = Input->pfValue[j - 1 + startPt];
				yesterday = Input->pfValue[j - 1 - moPeriod + startPt];
				
				if ((today - yesterday) > 0)
					CU[0] = CU[0] + (today - yesterday);
				else
					CD[0] = CD[0] + (yesterday - today);
			}
		}
		else // on subsequent passes, we simply take today's change up/down, and then smooth with wilder's method
		{
			today = Input->pfValue[i];
			yesterday = Input->pfValue[i-moPeriod];
			
			if ((today - yesterday) > 0)
				CU[0] = CU[0] + (today - yesterday);
			else
				CD[0] = CD[0] + (yesterday - today);
		}

		// Apply Wilder's Smoothing
		CU[0] = ( CU[1] * (n-1) + CU[0] ) / n; 
		CD[0] = ( CD[1] * (n-1) + CD[0] ) / n; 


		if (!IsDoubleZero(CU[0] + CD[0]))
			Result->pfValue[i] = (float)ForceFloatRange( 100 * (CU[0]  / (CU[0] + CD[0])) );
		else
			Result->pfValue[i] = 0.0;

		CU[1] = CU[0];
		CD[1] = CD[0];
	}	
}