/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

// here we assume that the period can not be less than 0, and if it is, it is set to be 0.
// this may not be ideal behavior if we want to be able to look into the future (less than 0).

// it is also different than the ROC, where we do not allow a period less than 1

// The periods for the Ref are slightly different than most indicators.
// A 1 period MA would reference the "today" bar only, whereas a 1 period
// ROC or Ref would reference the "yesterday" bar

void Ref (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
			MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int period;
	int curStartPt;

	Result->iFirstValid = startIdx;
	Result->iLastValid = endIdx;

	for (int i = startIdx; i <= endIdx; i++)
	{
			// **** NOTE ***** //
		// here we take the NEGATIVE of the period value, since the ref function in Metastock
		// works the reverse of the other indicators. (i.e. lookbacks are negative for ref() )
		period = int(ForceIntRange(-Periods->pfValue[i]));
		
		//if(period < 0) 
			//period = 0;
		
		curStartPt = i - period;

		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		// the extra check for the reverse, since we look ahead...
		if (curStartPt > endIdx)
		{
#ifdef TRUNC_RESULTS
			if (Result->iFirstValid > (i-1))
				Result->iLastValid = i - 1; // we only store it if it is lower than any previous
#endif

			curStartPt = endIdx;
		}

		Result->pfValue[i] = (float)ForceFloatRange(Input->pfValue[curStartPt]);
	}	
}
