/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

// here we assume that the period can not be less than 0, and if it is, it is set to be 0.
// this may not be ideal behavior if we want to be able to look into the future (less than 0).

// The periods for the Ref are slightly different than most indicators.
// A 1 period MA would reference the "today" bar only, whereas a 1 period
// ROC would reference the "yesterday" bar


// this function should require a special check to make sure the AssnArray is as large as the StartingArray
void RefWrite (const MSXDataInfoRec * StartingArray, const MSXDataInfoRec * AssnArray, 
			   const float constStartPos, const float constEndPos, MSXDataInfoRec * Result)
{
	float startPos = constStartPos;

	//  // NOTICE **** We switch from the negative input value to a positive one, because that's how I like to think of it!
	float endPos = constEndPos;

	int startIdx = StartingArray->iFirstValid;
	int endIdx = StartingArray->iLastValid;

	int curIdx;

	// the valid elements of the output are the same as the valid elements of the StartingArray
	Result->iFirstValid = startIdx;
	Result->iLastValid = endIdx;

	// To Start, we just set the Result to the StartingArray, then we change the values we need to.
	// note, this could probably be optimized into one loop...
	for (int i = startIdx; i <= endIdx; i++)
	{	
		Result->pfValue[i] = (float)ForceFloatRange(StartingArray->pfValue[i]);
	}

	// just a calculation speed safety precaution, so a user can't accendentally bog down the system...
	if (startPos > 99999)
		startPos = 99999;
	if (endPos > 99999)
		endPos = 99999;

	for (i = startPos; i <= endPos; i++)
	{	
		
		// not really a reason, but we start at the startPos.
		curIdx = endIdx + i;

		if (curIdx < startIdx)
		{
			// TRUNC_RESULTS WILL NOT WORK PROPERLY HERE, because it wasn't ever programmed....
			curIdx = startIdx;
		}
		if (curIdx > endIdx)
		{
			// TRUNC_RESULTS WILL NOT WORK PROPERLY HERE, because it wasn't ever programmed....
			curIdx = endIdx;
		}

		// this special case can occur when the AssnArray is smaller than the StartingArray,
		// we would like to return an MSX_ERROR, but that seems a little harsh, so we fill we an
		// obviously erroneous result value...
		if (curIdx < AssnArray->iFirstValid || curIdx > AssnArray->iLastValid)
			Result->pfValue[curIdx] = -99999;
		else
			Result->pfValue[curIdx] = (float)ForceFloatRange(AssnArray->pfValue[curIdx]);
	}

}
