/* 

/// Brad and Mike Adaptive Function:  STDERR (Standard Error)

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void STE (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				const double deviations, MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array

	int curStartPt; // the starting point calculated from the current main index and the current periods
	int prevStartPt; // the starting point calculated from the prev main index and the prev periods

	int period;	// Set to be the requested number of lookback indicies each time.
	int changePeriods; // The number (+ or -) change between prev period and cur period.
	
	double Sum = 0.0;
	double SumSquared = 0.0;
	double AvgSum;
	double AvgSumSquared;
	double tempDbl;

	Result->iFirstValid = startIdx; 
	Result->iLastValid = endIdx;

	curStartPt = startIdx;
	for (i = startIdx; i <= endIdx; i++)
	{
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		prevStartPt = curStartPt;
		curStartPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		tempDbl = Input->pfValue[i];
		Sum += tempDbl;
		tempDbl *= tempDbl;
		SumSquared += tempDbl;

		changePeriods = curStartPt - prevStartPt;

		if (changePeriods > 0)
		{
			for (int j = 0; j < changePeriods; j++)
			{	
				//Sum -= Input->pfValue[curStartPt - j - 1];
				
				tempDbl = Input->pfValue[curStartPt - j - 1];
				Sum -= tempDbl;
				tempDbl *= tempDbl;
				SumSquared -= tempDbl;
			}
		}
		else if (changePeriods < 0)
		{
			for (int j = 0; j > changePeriods; j--)
			{
				//Sum += Input->pfValue[prevStartPt + j - 1]; 
				
				tempDbl = Input->pfValue[prevStartPt + j - 1];
				Sum += tempDbl;
				tempDbl *= tempDbl;
				SumSquared += tempDbl;

			}
		}
		
		AvgSum = Sum / (double)(i - curStartPt + 1);  // (i - curStartPt + 1) = # of ACTUAL periods
		AvgSumSquared = SumSquared / (double)(i - curStartPt + 1);

		Result->pfValue[i] = (float)ForceFloatRange(AvgSumSquared - AvgSum * AvgSum);

		// This is the variance.
		tempDbl = Result->pfValue[i];

		// Now get the std dev.
        if( !IsDoubleZeroOrNeg(tempDbl) )
			Result->pfValue[i] = (float)ForceFloatRange(sqrt(tempDbl) * deviations);
		else
			Result->pfValue[i] = 0.0;

		// Now get the std err from the std dev.
		Result->pfValue[i] = (float)ForceFloatRange( Result->pfValue[i] / sqrt(i - curStartPt + 1) );

	}
}

