/* 

/// Brad and Mike Adaptive Function:  StdErrBandBot (Standard Error Band Bottom) )

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void STEbandBot (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
		   const MSXDataInfoRec * Periods, const float errors, MSXDataInfoRec * Result)
{
	MSXDataInfoRec Temp;

	// Return an empty array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &Temp) )
	{
		Result->iFirstValid = -1; 
		Result->iLastValid = 0;
		return;
	}

	LinearReg(Input, Periods, &Temp);
	STE(MasterData, Input, Periods, errors, Result);

	
	for(int j = Result->iFirstValid; j <= Result->iLastValid ; j++ )
		Result->pfValue[j] = (float)ForceFloatRange(Temp.pfValue[j]) - (Result->pfValue[j] * errors);
	
	// Here we assume that the LinearReg has a longer truncation than the 
	// stderr function, (they are currently the same, but could change!)

	// Don't Forget to DeAllocate Array
	MSX_ARRAY_DEALLOC(&Temp);
}
