/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*//* 

/// Brad and Mike Adaptive Function:  STDDEV (Standard Deviation)

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void Stdev (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				const double deviations, MSXDataInfoRec * Result)
{
	int startIdx = Input->iFirstValid;
	int endIdx = Input->iLastValid;

	int i; // the looping index for main array

	int curStartPt; // the starting point calculated from the current main index and the current periods
	int prevStartPt; // the starting point calculated from the prev main index and the prev periods

	int period;	// Set to be the requested number of lookback indicies each time.
	int changePeriods; // The number (+ or -) change between prev period and cur period.
	
	double Sum = 0.0;
	double SumSquared = 0.0;
	double AvgSum;
	double AvgSumSquared;
	double tempDbl;

	Result->iFirstValid = startIdx; 
	Result->iLastValid = endIdx;

	curStartPt = startIdx;
	for (i = startIdx; i <= endIdx; i++)
	{
		period = int(ForceIntRange(Periods->pfValue[i]));
		
		if(period < 1) 
			period = 1;
		
		prevStartPt = curStartPt;
		curStartPt = i - period + 1;

		// If we are attempting to look back farther than we are able, then this is not
		// producing a truely valid result.
		if (curStartPt < startIdx)
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			curStartPt = startIdx;
		}

		tempDbl = Input->pfValue[i];
		Sum += tempDbl;
		tempDbl *= tempDbl;
		SumSquared += tempDbl;

		changePeriods = curStartPt - prevStartPt;

		if (changePeriods > 0)
		{
			for (int j = 0; j < changePeriods; j++)
			{	
				//Sum -= Input->pfValue[curStartPt - j - 1];
				
				tempDbl = Input->pfValue[curStartPt - j - 1];
				Sum -= tempDbl;
				tempDbl *= tempDbl;
				SumSquared -= tempDbl;
			}
		}
		else if (changePeriods < 0)
		{
			for (int j = 0; j > changePeriods; j--)
			{
				//Sum += Input->pfValue[prevStartPt + j - 1]; 
				
				tempDbl = Input->pfValue[prevStartPt + j - 1];
				Sum += tempDbl;
				tempDbl *= tempDbl;
				SumSquared += tempDbl;

			}
		}
		
		AvgSum = Sum / (double)(i - curStartPt + 1);  // (i - curStartPt + 1 = # of ACTUAL periods
		AvgSumSquared = SumSquared / (double)(i - curStartPt + 1);

		Result->pfValue[i] = (float)ForceFloatRange(AvgSumSquared - AvgSum * AvgSum);

		// This is the variance.
		tempDbl = Result->pfValue[i];

		// Now get the std dev.
        if( !IsDoubleZeroOrNeg(tempDbl) )
			Result->pfValue[i] = (float)ForceFloatRange(sqrt(tempDbl) * deviations);
		else
			Result->pfValue[i] = 0.0;

	}
}