/* 

/// Brad and Mike Adaptive Function:  StochOsc (Stochastic Oscillator)

  //  12-20-04 - Non Optomized, mainly just Umit and Steve's Code

*/

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"


void Stoch(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
			MSXDataInfoRec * Result)
{
	// create temp variable on the stack.
	MSXDataInfoRec TempHHV;

	// Return an invalid array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &TempHHV) )
	{
		Result->iFirstValid = 0; 
		Result->iLastValid = 0;
		return;
	}

	// NOTE : we are using Result as temp array here.
	// bug fixed on 5-10-05.  sClose changed to sHigh and sLow for Inputs to HHV and LLV
	HHV(&MasterData->sHigh, Periods, &TempHHV);
	LLV(&MasterData->sLow, Periods, Result);
	
	int startIdx = Result->iFirstValid;
	int endIdx = Result->iLastValid;

	for (int i = startIdx; i <= endIdx; i++)
	{

		if (!IsDoubleZero(TempHHV.pfValue[i] - Result->pfValue[i]))
		{
			Result->pfValue[i] = (float)ForceFloatRange(
				
				100 * (MasterData->sClose.pfValue[i] - Result->pfValue[i])
				/ 
				(TempHHV.pfValue[i] - Result->pfValue[i])

				//Result->pfValue[i] + ((TempHHV.pfValue[i] - Result->pfValue[i]) / 2.0 )
				
				); 
		}
		else
			Result->pfValue[i] = 0.0;
	}

	MSX_ARRAY_DEALLOC(&TempHHV);
}



	/*
	int a_iKSlowing = (int)ForceIntRange(a_fKSlowing);

	if(a_iKSlowing < 1) a_iKSlowing = 1;

	int l_iIndex = MasterData->sClose.iFirstValid;
	int l_iMaxIndex = MasterData->sClose.iLastValid;
	double l_lfDivident = 0.0;
	double l_lfDivisor = 0.0;
	int l_iPeriod = 0;
	float l_fLLow = 0;
	float l_fHHigh = 0;
	double l_lfDbl = 0.0;

	Result->iFirstValid = l_iIndex + a_iKSlowing;
	Result->iLastValid = l_iMaxIndex;

	for(int i= ForceIntRange(l_iIndex + a_iKSlowing); i<= l_iMaxIndex; i++)
	{
		l_iPeriod = (int)ForceIntRange(Periods->pfValue[i]);

		if(l_iPeriod < 1) l_iPeriod = 1;

		if(l_iPeriod > i) 
		{
#ifdef TRUNC_RESULTS
			Result->iFirstValid = i + 1;
			
			// if we don't do this, we would be returning an illegal array, but we want
			// to instead return an empty array.
			if (i+1 > Result->iLastValid)
				Result->iFirstValid = i;
#endif
			l_iPeriod = i;
		}

		l_lfDivident = 0.0;
		l_lfDivisor = 0.0;
		for(int y=i; y > ForceIntRange(i - a_iKSlowing); y--)
		{
			l_fLLow = FLT_MAX;
			l_fHHigh = FLT_MIN;
			for (int x=y; x > ForceIntRange(y - l_iPeriod); x--)
			{
				l_fLLow = (l_fLLow > MasterData->sLow.pfValue[x])? MasterData->sLow.pfValue[x] :l_fLLow;
				l_fHHigh = (l_fHHigh < MasterData->sHigh.pfValue[x])? MasterData->sHigh.pfValue[x] :l_fHHigh;
			}

			l_lfDivident += MasterData->sClose.pfValue[y] - l_fLLow;
			l_lfDivisor += l_fHHigh - l_fLLow;
		}

		if(l_lfDivisor == 0) //l_lfDivisor = FLT_MIN;
			Result->pfValue[i] = 0.0;
		else
		{
			l_lfDbl = (l_lfDivident/l_lfDivisor) * 100;
			l_lfDbl = ForceFloatRange(l_lfDbl);
		
			Result->pfValue[i] = float(l_lfDbl);
		}
	}
	*/