/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TA-Lib BSD License:
TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

*/
	 /* For an explanation of this function, please read:
    * 
    * Stocks & Commodities V. 12:1 (11-19): 
    *   Smoothing Data With Faster Moving Averages
    * Stocks & Commodities V. 12:2 (72-80): 
    *   Smoothing Data With Less Lag
    *
    * Both magazine articles written by Patrick G. Mulloy
    *
    * Essentially, a TEMA of time serie 't' is:
    *   EMA1 = EMA(t,period)
    *   EMA2 = EMA(EMA(t,period),period)
    *   EMA3 = EMA(EMA(EMA(t,period),period))
    *   TEMA = 3*EMA1 - 3*EMA2 + EMA3
    *
    * TEMA offers a moving average with less lags then the
    * traditional EMA.
    *
    * Do not confuse a TEMA with EMA3. Both are called "Triple EMA"
    * in the litterature.
    *
    * DEMA is very similar (and from the same author).
    */

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

/*

e1 := ExtFml("ASI.EMA",CLOSE,10);
e2 := ExtFml("ASI.EMA",e1,10);
e3 := ExtFml("ASI.EMA",e2,10);

tma := 3*e1 - 3*e2 + e3;

*/

void Tema (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				 MSXDataInfoRec * Result)

{
	MSXDataInfoRec Temp1;
	MSXDataInfoRec Temp2;

	if (! MSX_ARRAY_ALLOC(MasterData, &Temp1) )
	{
		Result->iFirstValid = -1; 
		Result->iLastValid = 0;
		return;
	}

	if (! MSX_ARRAY_ALLOC(MasterData, &Temp2) )
	{
		Result->iFirstValid = -1; 
		Result->iLastValid = 0;
		return;
	}

	// Call the EMA function and then pass the output of first as input to second.
	// We can use the results array as a temp array.
	EMA(Input, Periods, &Temp1);
	EMA(&Temp1, Periods, &Temp2);
	EMA(&Temp2, Periods, Result);

	for(int j = Result->iFirstValid; j <= Result->iLastValid ; j++ )
		Result->pfValue[j] = (float)ForceFloatRange(
			3.0 * Temp1.pfValue[j] 
			- 3.0 * Temp2.pfValue[j] 
			+ Result->pfValue[j]);
	
	// Since the Result Array was the last one returned from another function,
	// It contains the correct FirstValid and LastValid values.

	// Don't Forget to DeAllocate Array
	MSX_ARRAY_DEALLOC(&Temp1);
	MSX_ARRAY_DEALLOC(&Temp2);
}