#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "MSXSafetyChecks.h"
#include "MSXStruc.h"
#include "Adaptive.h"

void TRIX (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
		   const MSXDataInfoRec * Periods, MSXDataInfoRec * Result)

{
	MSXDataInfoRec Temp;

	// Return an invalid array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &Temp) )
	{
		Result->iFirstValid = 0; 
		Result->iLastValid = 0;
		return;
	}

	MSXDataInfoRec ConstPeriods;

	// Return an invalid array if we cannot allocate memory on the heap for the Temp array.
	if (! MSX_ARRAY_ALLOC(MasterData, &ConstPeriods) )
	{
		Result->iFirstValid = 0; 
		Result->iLastValid = 0;
		return;
	}

	for (int i = ConstPeriods.iFirstValid; i <= ConstPeriods.iLastValid; i++)
		ConstPeriods.pfValue[i] = 2.0;
	
	EMA(Input, Periods, &Temp);
	
	EMA(&Temp, Periods, Result);

	EMA(Result, Periods, &Temp);

	for (i = Temp.iFirstValid+1; i <= Temp.iLastValid; i++)
	{
		if (!IsDoubleZero(Input->pfValue[i-1]))
			Result->pfValue[i] = (float)ForceFloatRange(
								((Temp.pfValue[i] / Temp.pfValue[i-1]) - 1) * 100
								);
		else
			Result->pfValue[i] = 0.0;
	}

	Result->iFirstValid = Temp.iFirstValid;
	Result->iLastValid = Temp.iLastValid;

	// l bar percent ROC
	// need to program a fill function to fill array with constant 1.
	//ROC(&Temp, &ConstPeriods, 0, Result);

	
	MSX_ARRAY_DEALLOC(&Temp);
}