#ifndef ADAPTIVE_H
#define ADAPTIVE_H

/*
This is a copy of the DML BSD License:

Dynamic Market Lab, LLC Copyright (c) 2004-2005
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

- Neither name of author nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

// This is TA-Lib's license, and parts of TA-Lib are used in the code, so we show it here...

/*
This is a copy of the TA-Lib BSD License:

TA-LIB Copyright (c) 1999-2005, Mario Fortier
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

- Neither name of author nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================
TA-LIB IS EMBEDDING OTHER OPEN-SOURCE PROJECTS. ALL THEIR LICENSES ARE INCLUDED BELOW. THESE LICENSES ARE ALSO MADE AVAILABLE IN THE PACKAGE DISTRIBUTED AT HTTP://TA-LIB.ORG
=========================================================
*/







//////////////////////////////////////////////
//**********

// This file is used to internally link function to other functions.

// Any file "including" this file inside of it will have access to 
// all the functions, data members and structure definitions "externed" here and located in other files.

//***********
///////////////////////////////////////////////////////////////////////////////

#include "MSXStruc.h"


// This is a defined value that determines whether or not a function truncates the size of
// the return array.  If it is defined, then code in the functions
// is called to adjust the iFirstValid member of the function's Returned Array

#define TRUNC_RESULTS 1; 

#define DEMO_VERSION 1;

/// NOTE : Some of the functions that are "model" functions are listed below, and
//			the contain extra commenting to describe the algorithm, but the comments are
//			removed in similar redundant functions.

/// WMA - example of "simple" double loop algorithm
// AroonDown - example of "sliding window" loop algorithm (faster)


extern void SMA (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);

extern void EMA (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);

extern void WMA (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);


extern void AroonDown (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);

extern void AroonUp (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);

extern void ADX(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void ATR(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void BBandBot (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
				const MSXDataInfoRec * Periods, const float deviations, MSXDataInfoRec * Result);

extern void BBandTop (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
				const MSXDataInfoRec * Periods, const float deviations, MSXDataInfoRec * Result);

extern void CCI(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void Correl (const MSXDataInfoRec * Input1, const MSXDataInfoRec * Input2, 
			 const MSXDataInfoRec * Periods, MSXDataInfoRec * Result);

extern void DEMA (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
				const MSXDataInfoRec * Periods, MSXDataInfoRec * Result);

extern void DM_Plus (const MSXDataRec * MasterData, MSXDataInfoRec * Result);
extern void DM_Minus (const MSXDataRec * MasterData, MSXDataInfoRec * Result);

/*
extern void DPO(const MSXDataRec * MasterData, const MSXDataInfoRec * Input,
				const MSXDataInfoRec * Periods, MSXDataInfoRec * Result);		
*/
extern void DX(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void ADXR(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void EffIndex(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
							MSXDataInfoRec * Result);

extern void ForecastOsc (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
						MSXDataInfoRec * Result);

extern void HHV (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);

extern void HHVBarsAgo (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);

extern void LinearReg (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
			MSXDataInfoRec * Result);

extern void LinearRegSlope (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
			MSXDataInfoRec * Result);

extern void LLV (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);

extern void LLVBarsAgo (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 MSXDataInfoRec * Result);

extern void MACD (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
			const MSXDataInfoRec * slowPeriods, const MSXDataInfoRec * fastPeriods, MSXDataInfoRec * Result);

extern void MDI(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void Mid (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
						MSXDataInfoRec * Result);

extern void Mo (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
						MSXDataInfoRec * Result);

extern void Mov (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
		   const MSXDataInfoRec * Periods, const int avgType, MSXDataInfoRec * Result);

extern void PDI(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void PriceChannelHigh (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void PriceChannelLow (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void ProjBandBot (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void ProjBandTop (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
					MSXDataInfoRec * Result);

extern void ProjOsc (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
						MSXDataInfoRec * Result);

extern void ROC (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
				 const int type, MSXDataInfoRec * Result);

extern void Ref (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				MSXDataInfoRec * Result);

extern void RefWrite (const MSXDataInfoRec * StartingArray, const MSXDataInfoRec * AssnArray, 
			   const float startPos, const float endPos, MSXDataInfoRec * Result);

extern void RSquared (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
						MSXDataInfoRec * Result);

extern void RMI (const MSXDataInfoRec * Input, const MSXDataInfoRec * RegPeriods, 
				 const MSXDataInfoRec * MoPeriods, MSXDataInfoRec * Result);

extern void RSI (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				 MSXDataInfoRec * Result);

extern void RVGI (const MSXDataRec * MasterData, const MSXDataInfoRec * Input,
				 const MSXDataInfoRec * Periods,	MSXDataInfoRec * Result);

extern void Stdev (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
					const MSXDataInfoRec * Periods, const double deviations, MSXDataInfoRec * Result);

extern void STE (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
					const MSXDataInfoRec * Periods, const double deviations, MSXDataInfoRec * Result);

extern void STEbandBot (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
					const MSXDataInfoRec * Periods, const float errors, MSXDataInfoRec * Result);

extern void STEbandTop (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
					const MSXDataInfoRec * Periods, const float errors, MSXDataInfoRec * Result);

extern void StochMomentum (const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
						MSXDataInfoRec * Result);

extern void Stoch(const MSXDataRec * MasterData, const MSXDataInfoRec * Periods,
				MSXDataInfoRec * Result);

extern void Sum (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
						MSXDataInfoRec * Result);

extern void Tema (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
				  const MSXDataInfoRec * Periods, MSXDataInfoRec * Result);

extern void TrueRange (const MSXDataRec * MasterData, MSXDataInfoRec * Result);

extern void TSF (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods,
						MSXDataInfoRec * Result);

extern void TRIX (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
				  const MSXDataInfoRec * Periods, MSXDataInfoRec * Result);

extern void VAR (const MSXDataInfoRec * Input, const MSXDataInfoRec * Periods, 
				MSXDataInfoRec * Result);


extern void CyclePeriod_C (const MSXDataRec * MasterData, const MSXDataInfoRec * Input, 
						   const MSXDataInfoRec * alphas, const MSXDataInfoRec * DeltaPhaseLows, 
						   const MSXDataInfoRec * DeltaPhaseHighs, MSXDataInfoRec * Result);	

extern BOOL CalculateHilbertPeriod(const float *a_pfPrice, const int a_iFirstValid,
						    const int a_iLastValid, float *a_pfPeriod, const int a_iArraySize);

// Internal Functions

	// Single Bar Functions

	extern double MSX_TrueRange(double TH, double TL, double YC);
	extern double MSXMax(double d1, double d2);
	extern double MSXMin(double d1, double d2);
	extern double MSX_DM_Plus(double A, double B, double C, double D); 
	extern double MSX_DM_Minus(double A, double B, double C, double D); 

#endif