/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

// The Code below is Adapted from the Metastock Developers Kit Manual

#include <string.h>
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#include "Adaptive.h"
#include "MSXSafetyChecks.h"
#include "MSXStruc.h"  // required definitions

#include "license.h"

// we don't want C++ name-mangling
#ifdef __cplusplus
extern "C" {
#endif

//===============================================================================
// MetaStock MSX dll Functions
// 
// Each external function available to Metastock through MSX is defined here.

//===============================================================================

// ************************* ******************************* **********

	// AroonDown

// ************************* ***************** *************************

BOOL __stdcall AroonDown (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{

	// relic of debug testing for license file checker problem - 7-12-05.
	/*
	int typeASI = 0;
	int ret = IsLoginSuccessful(typeASI);

	if (ret != 0)
	{
		//char str[10];
		//itoa(ret, str, 10);
		//str[10] = '\0';

		//MessageBoxA(NULL, str, "The DML", MB_OK);
		//ShellExecute(NULL,"open", "http://www.thedml.com/expired", "", "c:\\", SW_SHOWNORMAL);
		return MSX_ERROR;
	}
*/

	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		// According to the MetaStock Developer's Kit, we return a "quiet" error when we 
		// receive an invalid input.
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	// Always check for the correct count for all four input argument arrays
	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	AroonDown(Input1, Input2, ResultRec->psResultArray);

	// error message is returned via the ResultRec, if it is not empty, we had an error.
	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}

	// always check to make sure the Result is legal, if it is not, return empty result and error.
	// This is not required, but is a last chance safety effort to catch corrupted return data.
	// (for example, MetaStock says that writing a result past the sClose.iLastValid is really bad.)
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}

	return MSX_SUCCESS;
}


// ************************* ******************************* **********

	// AroonUp

// ************************* ***************** *************************

BOOL __stdcall AroonUp (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	AroonUp(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// ADX

// ************************* ***************** *************************

BOOL __stdcall ADX (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	ADX(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// ATR

// ************************* ***************** *************************

BOOL __stdcall ATR (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	ATR(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// BBandBot

// ************************* ***************** *************************

BOOL __stdcall BBandBot (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 1 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	BBandBot(MasterData, Input1, Input2, NumericArgs->fNumerics[0], ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// BBandTop

// ************************* ***************** *************************

BOOL __stdcall BBandTop (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 1 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	BBandTop(MasterData, Input1, Input2, NumericArgs->fNumerics[0], ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// CCI (Commodity Channel Index)

// ************************* ***************** *************************

BOOL __stdcall CCI (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	CCI(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Correl

// ************************* ***************** *************************

BOOL __stdcall Correl (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;
	const MSXDataInfoRec * Input3;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];
	Input3 = DataInfoArgs->psDataInfoRecs[2];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) 
		&& IsLegalMSXArray(MasterData, Input3) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 3 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	Correl(Input1, Input2, Input3, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// DEMA (Double Exponential Moving Avergage)

// ************************* ***************** *************************

BOOL __stdcall DEMA (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	DEMA(MasterData, Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// DX

// ************************* ***************** *************************

BOOL __stdcall DX (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	DX(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// ADXR

// ************************* ***************** *************************

BOOL __stdcall ADXR (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	ADXR(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// EffIndex (Efficiency Index)

// ************************* ***************** *************************

BOOL __stdcall EffIndex (const MSXDataRec * MasterData,
						const MSXDataInfoRecArgsArray * DataInfoArgs,
						const MSXNumericArgsArray * NumericArgs,
						const MSXStringArgsArray * StringArgs,
						const MSXCustomArgsArray * CustomArgs,
						MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	EffIndex(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// EMA

// ************************* ***************** *************************

BOOL __stdcall EMA (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	EMA(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}


// ************************* ******************************* **********

	// ForecastOsc

// ************************* ***************** *************************

BOOL __stdcall ForecastOsc (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	ForecastOsc(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// HHV (Highest High Value)

// ************************* ***************** *************************

BOOL __stdcall HHV (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	//for (int i = 0; i < 10000; i++)
	//{/// Main Internal Function Call
		HHV(Input1, Input2, ResultRec->psResultArray);
	//}

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}


// ************************* ******************************* **********

	// HHVBarsAgo

// ************************* ***************** *************************

BOOL __stdcall HHVBarsAgo (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	HHVBarsAgo(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// LinearReg

// ************************* ***************** *************************

BOOL __stdcall LinearReg (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	LinearReg(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// LinearReg Slope

// ************************* ***************** *************************

BOOL __stdcall LinearRegSlope (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	LinearRegSlope(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}
// ************************* ******************************* **********

	// LLV (Lowest Low Value)

// ************************* ***************** *************************

BOOL __stdcall LLV (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	LLV(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// LLVBarsAgo

// ************************* ***************** *************************

BOOL __stdcall LLVBarsAgo (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	LLVBarsAgo(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}


// ************************* ******************************* **********

	// MACD  (Moving Average Convergence/Divergence)

// ************************* ***************** *************************

BOOL __stdcall MACD (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;
	const MSXDataInfoRec * Input3;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];
	Input3 = DataInfoArgs->psDataInfoRecs[2];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) && IsLegalMSXArray(MasterData, Input3)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 3 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	MACD(MasterData, Input1, Input2, Input3, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// MDI

// ************************* ***************** *************************

BOOL __stdcall MDI (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	MDI(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Mid

// ************************* ***************** *************************

BOOL __stdcall Mid (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	Mid(MasterData, Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Mo

// ************************* ***************** *************************

BOOL __stdcall Mo (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	Mo(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// PDI

// ************************* ***************** *************************

BOOL __stdcall PDI (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	PDI(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// PriceChannelHigh

// ************************* ***************** *************************

BOOL __stdcall PriceChannelHigh (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	PriceChannelHigh(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// PriceChannelLow

// ************************* ***************** *************************

BOOL __stdcall PriceChannelLow (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	PriceChannelLow(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// ProjBandBot

// ************************* ***************** *************************

BOOL __stdcall ProjBandBot (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	ProjBandBot(MasterData, Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// ProjBandTop

// ************************* ***************** *************************

BOOL __stdcall ProjBandTop (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	ProjBandTop(MasterData, Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// ProjOsc

// ************************* ***************** *************************

BOOL __stdcall ProjOsc (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	ProjOsc(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// RSquared

// ************************* ***************** *************************

BOOL __stdcall RSquared (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	RSquared(MasterData, Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// ROC

// ************************* ***************** *************************

BOOL __stdcall ROC (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	
	//const int avgType = CustomArgs->iCustomIDs[0];
	const int avgType = (int)NumericArgs->fNumerics[0];

	if (avgType != 0 && avgType != 1)
	{
		/*
		strncpy (ResultRec->szExtendedError, "Input Error: Invalid Custom Arg. Must be one of: '0', '1'",  // '%','$','Percent','Absolute'",
					sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
		*/

		// return quiet error, empty result, instead
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = 0;

		return MSX_SUCCESS;
	}


	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 1 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	ROC(Input1, Input2, avgType, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Ref

// ************************* ***************** *************************

BOOL __stdcall Ref (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	Ref(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Ref

// ************************* ***************** *************************

BOOL __stdcall RefWrite (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;
	//const MSXDataInfoRec * Input3;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];
	//Input3 = DataInfoArgs->psDataInfoRecs[2];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 2 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}

	// Instead of returning a "real" error here, we return a "quiet" error in the function.
/*
	if (Input1->iFirstValid > Input2->iFirstValid || Input1->iLastValid < Input2->iLastValid)
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: Assignment Array is too small",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
*/
	
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	RefWrite(Input1, Input2, NumericArgs->fNumerics[0], NumericArgs->fNumerics[1], ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}
// ************************* ******************************* **********

	// RMI

// ************************* ***************** *************************

BOOL __stdcall RMI (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;
	const MSXDataInfoRec * Input3;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];
	Input3 = DataInfoArgs->psDataInfoRecs[2];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) 
		&& IsLegalMSXArray(MasterData, Input3) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 3 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	RMI(Input1, Input2, Input3, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// RSI

// ************************* ***************** *************************

BOOL __stdcall RSI (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	RSI(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// RVGI

// ************************* ***************** *************************

BOOL __stdcall RVGI (const MSXDataRec * MasterData,
					const MSXDataInfoRecArgsArray * DataInfoArgs,
					const MSXNumericArgsArray * NumericArgs,
					const MSXStringArgsArray * StringArgs,
					const MSXCustomArgsArray * CustomArgs,
					MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	RVGI(MasterData, Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// SMA

// ************************* ***************** *************************

BOOL __stdcall SMA (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	for (int i =0; i < 1000; i++)
	{
		/// Main Internal Function Call
		SMA(Input1, Input2, ResultRec->psResultArray);
	}

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Stdev

// ************************* ***************** *************************

BOOL __stdcall Stdev (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 1 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	Stdev(MasterData, Input1, Input2, NumericArgs->fNumerics[0], ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// STE

// ************************* ***************** *************************

BOOL __stdcall STE (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 1 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	STE(MasterData, Input1, Input2, NumericArgs->fNumerics[0], ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// STEbandBot

// ************************* ***************** *************************

BOOL __stdcall STEbandBot (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 1 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	STEbandBot(MasterData, Input1, Input2, NumericArgs->fNumerics[0], ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// STEbandTop

// ************************* ***************** *************************

BOOL __stdcall STEbandTop (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 1 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	STEbandTop(MasterData, Input1, Input2, NumericArgs->fNumerics[0], ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// StochMomentum

// ************************* ***************** *************************

BOOL __stdcall StochMomentum (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1)))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	StochMomentum(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Stoch

// ************************* ***************** *************************


BOOL __stdcall Stoch (const MSXDataRec * MasterData,
						const MSXDataInfoRecArgsArray * DataInfoArgs,
						const MSXNumericArgsArray * NumericArgs,
						const MSXStringArgsArray * StringArgs,
						const MSXCustomArgsArray * CustomArgs,
						MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;

	Input1 = DataInfoArgs->psDataInfoRecs[0];

	if (!(IsLegalMSXArray(MasterData, Input1) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 1 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	Stoch(MasterData, Input1, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Sum

// ************************* ***************** *************************

BOOL __stdcall Sum (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	Sum(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// Tema

// ************************* ***************** *************************

BOOL __stdcall Tema (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	Tema(MasterData, Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// True Range

// ************************* ***************** *************************

BOOL __stdcall TrueRange (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{

	if (! (DataInfoArgs->iNRecs == 0 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	TrueRange(MasterData, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}


// ************************* ******************************* **********

	// TSF

// ************************* ***************** *************************

BOOL __stdcall TSF (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	TSF(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// TRIX

// ************************* ***************** *************************

BOOL __stdcall TRIX (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	TRIX(MasterData, Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// VAR

// ************************* ***************** *************************

BOOL __stdcall VAR (const MSXDataRec * MasterData,
                          const MSXDataInfoRecArgsArray * DataInfoArgs,
                          const MSXNumericArgsArray * NumericArgs,
                          const MSXStringArgsArray * StringArgs,
                          const MSXCustomArgsArray * CustomArgs,
                          MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	VAR(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}

// ************************* ******************************* **********

	// WMA

// ************************* ***************** *************************

BOOL __stdcall WMA (const MSXDataRec * MasterData,
                    const MSXDataInfoRecArgsArray * DataInfoArgs,
                    const MSXNumericArgsArray * NumericArgs,
                    const MSXStringArgsArray * StringArgs,
                    const MSXCustomArgsArray * CustomArgs,
                    MSXResultRec * ResultRec)
{
	const MSXDataInfoRec * Input1;
	const MSXDataInfoRec * Input2;

	Input1 = DataInfoArgs->psDataInfoRecs[0];
	Input2 = DataInfoArgs->psDataInfoRecs[1];

	if (!(IsLegalMSXArray(MasterData, Input1) && IsLegalMSXArray(MasterData, Input2) ))
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_SUCCESS;
	}

	if (! (DataInfoArgs->iNRecs == 2 && NumericArgs->iNRecs == 0 && 
		StringArgs->iNRecs == 0 && CustomArgs->iNRecs == 0))
	{		
		strncpy (ResultRec->szExtendedError, "Input Error: wrong number of arguments",
						sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;

		return MSX_ERROR;
	}
		
	*ResultRec->szExtendedError = NULL;

	/// Main Internal Function Call
	WMA(Input1, Input2, ResultRec->psResultArray);

	if (*ResultRec->szExtendedError != NULL)
	{
		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
		return MSX_ERROR;
	}
	if (!IsLegalMSXArray(MasterData, ResultRec->psResultArray))
	{
		strncpy (ResultRec->szExtendedError, "Internal Error: Corrupted Result Array",
			sizeof(ResultRec->szExtendedError)-1);

		ResultRec->psResultArray->iFirstValid = 0;
		ResultRec->psResultArray->iLastValid = -1;
			
		return MSX_ERROR;
	}
	return MSX_SUCCESS;
}


#ifdef __cplusplus
}
#endif
