/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

// The Code below is Adapted from the Metastock Developers Kit Manual////

//// Include this file if you are interfacing with MetaStock MSX DLL

/////
#include <float.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>

#include "MSXStruc.h"
#include "Adaptive.h" 


double ForceFloatRange (double a_lfDbl)
{
	if (a_lfDbl > 0.0)
	{
		a_lfDbl = MSXMin (a_lfDbl, double(FLT_MAX)); // make sure pos number <= FLT_MAX
		a_lfDbl = MSXMax (a_lfDbl, double(FLT_MIN)); // make sure pos number >= FLT_MIN
	}
	else
	{
		if (a_lfDbl < 0.0)
		{
			a_lfDbl = MSXMax (a_lfDbl, double(-FLT_MAX)); // make sure neg number >= -FLT_MAX
			a_lfDbl = MSXMin (a_lfDbl, double(-FLT_MIN)); // make sure neg number <= -FLT_MIN
		}
	}
	return a_lfDbl;
}

double ForceIntRange (double a_lfDbl)
{
	
	if (a_lfDbl > 0.0)
		a_lfDbl = MSXMin (a_lfDbl, double(INT_MAX - 1)); 

	else if (a_lfDbl < 0.0) 
		a_lfDbl = MSXMax (a_lfDbl, double(INT_MIN + 1)); 

	return a_lfDbl;
}

// Checks to see if a float can be converted to an int without any truncation (other than the decimals, of course)
BOOL IsInIntRange (float a_fFloat)
{
	if (a_fFloat > 0.0)
		if (a_fFloat > (float)INT_MAX)
			return FALSE;
		
	if (a_fFloat < 0.0)
		if (a_fFloat < (float)INT_MIN)
			return FALSE;

	return TRUE;
}

BOOL IsDoubleZero(double tempDbl)
{
	if (tempDbl == 0)
		return TRUE;
	else
		return FALSE;

}

BOOL IsDoubleZeroOrNeg(double tempDbl)
{
	if ( (tempDbl == 0) || (tempDbl < 0.0) )
		return TRUE;
	else
		return FALSE;

}

// IsLegalMSXArray is used to test incoming data array arguments and the outgoing result array for compliance with 
// MetaStock standards.

// It should be called with the one and only MSXDataRec pointer as the first argument.
//  The second argument can be an input array, the result array, or one of the sub array members of the MSXDataRec struct.

// This function is used to make sure that the valid indicies of the argument array are not beyond those of the closing price
// in Metastocks MasterData array that is available to all functions.
// AND that the lower and upper bounds are all above zero, also according to MetaStock.
IsLegalMSXArray(const MSXDataRec * MasterData, const MSXDataInfoRec * ArgData)
{ 
	if (ArgData->iFirstValid < 0)
		return FALSE;
	if (ArgData->iLastValid < 0)
		return FALSE;
	if (ArgData->iLastValid < ArgData->iFirstValid)
		return FALSE;
	if (ArgData->iFirstValid < MasterData->sClose.iFirstValid)
		return FALSE;
	if (ArgData->iLastValid > MasterData->sClose.iLastValid)
		return FALSE;

	return TRUE;
}

MSX_ARRAY_ALLOC (const MSXDataRec * MasterData, MSXDataInfoRec * ArgData)
{
	ArgData->iFirstValid = MasterData->sClose.iFirstValid;
	ArgData->iLastValid = MasterData->sClose.iLastValid;

	// return FALSE if the MasterData was bogus.
	if (ArgData->iFirstValid < 0)
		return FALSE;
	if (ArgData->iLastValid < 0)
		return FALSE;
	if (ArgData->iLastValid <= ArgData->iFirstValid)
		return FALSE;

	// allocate memory, fixed bug here on 5-9-05, a big bug, before it was fixed
	// we had potential for access violations, and could not safely use an allocated
	// array as a argument to an internal function.

	//ArgData->pfValue = new float[ ArgData->iLastValid - ArgData->iFirstValid + 1];
	ArgData->pfValue = new float[ ArgData->iLastValid + 1];

	// This is the only way we check for memory alloc errors, probably could be more robust.
	if (ArgData->pfValue == NULL)
		return FALSE;
	else
		return TRUE;
}

MSX_ARRAY_DEALLOC (MSXDataInfoRec * ArgData)
{
	delete[] ArgData->pfValue;

	return TRUE;
}

void ZeroMemory ( double * array, int size)
{
	memset(array, 0, size);
}