#ifndef MSXSAFETYCHECKS_H
#define MSXSAFETYCHECKS_H

/*

DML BSD License

  /* The Dynamic Market Lab, LLC Copyright (c) 2004-2005,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither name of author nor the names of its contributors
 *   may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

// The Code below is Adapted from the Metastock Developers Kit Manual

// ************************************************************************
// These local utility functions are used to help ensure that no overflows
//   or underflows will occur during calcuations.  The MSXTest program
//   Stress Test function will call your DLL with a wide range of values,
//   including positive and negative values of FLT_MAX and FLT_MIN.
// Perform all intermediate calculations using doubles and then force the
//   results into the range of a float.
// Locally defined macros are used to avoid compiler definition differences.

// IsLegalMSXArray is used to test incoming data array arguments and the outgoing result 
//array for compliance with MetaStock standards.
// It should be called with the one and only MSXDataRec pointer as the first argument.
//  The second argument can be an input array, the result array, or one of the sub array members of the MSXDataRec struct.
// This function is used to make sure that the valid indicies of the argument array are not beyond those of the price array
// AND that the lower and upper bounds are all above zero, and this according to MetaStock.

// ************************************************************************
#ifndef BOOL
typedef int BOOL;
#endif

#include "MSXStruc.h"

extern double ForceFloatRange (double dbl);
extern double ForceIntRange (double dbl);
extern BOOL IsInIntRange (float flt);
extern BOOL IsDoubleZero(double tempDbl);
extern BOOL IsDoubleZeroOrNeg(double dbl);
extern BOOL IsLegalMSXArray(const MSXDataRec * PriceData, const MSXDataInfoRec * ArgData);
extern BOOL MSX_ARRAY_ALLOC (const MSXDataRec * MasterData, MSXDataInfoRec * ArgData);
extern BOOL MSX_ARRAY_DEALLOC (MSXDataInfoRec * ArgData);


extern void ZeroMemory ( double * array, int size);

#endif